@@ -982,7 +982,7 @@ set_number_of_spaces_hint (MetaScreen *screen,
 
   data[0] = n_spaces;
 
-  meta_verbose ("Setting _NET_NUMBER_OF_DESKTOPS to %ld\n", data[0]);
+  meta_verbose ("Setting _NET_NUMBER_OF_DESKTOPS to %lu\n", data[0]);
 
   meta_error_trap_push (screen->display);
   XChangeProperty (screen->display->xdisplay, screen->xroot,
@@ -1003,7 +1003,7 @@ set_desktop_geometry_hint (MetaScreen *screen)
   data[0] = screen->rect.width;
   data[1] = screen->rect.height;
 
-  meta_verbose ("Setting _NET_DESKTOP_GEOMETRY to %ld, %ld\n", data[0], data[1]);
+  meta_verbose ("Setting _NET_DESKTOP_GEOMETRY to %lu, %lu\n", data[0], data[1]);
 
   meta_error_trap_push (screen->display);
   XChangeProperty (screen->display->xdisplay, screen->xroot,
@@ -1196,38 +1196,50 @@ meta_screen_ensure_tab_popup (MetaScreen      *screen,
       entries[i].demands_attention = window->wm_state_demands_attention;
       
       if (show_type == META_TAB_SHOW_INSTANTLY ||
-          !window->minimized                   ||
+          !entries[i].hidden                   ||
           !meta_window_get_icon_geometry (window, &r))
         meta_window_get_outer_rect (window, &r);
       
       entries[i].rect = r;
 
-      /* Find inside of highlight rectangle to be used
-       * when window is outlined for tabbing.
-       * This should be the size of the east/west frame,
-       * and the size of the south frame, on those sides.
-       * on the top it should be the size of the south frame
+      /* Find inside of highlight rectangle to be used when window is
+       * outlined for tabbing.  This should be the size of the
+       * east/west frame, and the size of the south frame, on those
+       * sides.  On the top it should be the size of the south frame
        * edge.
        */
-      if (window->frame)
-        {
-          int south = window->frame->rect.height - window->frame->child_y -
-            window->rect.height;
-          int east = window->frame->child_x;
-          entries[i].inner_rect.x = east;
-          entries[i].inner_rect.y = south;
-          entries[i].inner_rect.width = window->rect.width;
-          entries[i].inner_rect.height = window->frame->rect.height - south * 2;
-        }
-      else
-        {
-          /* Use an arbitrary border size */
 #define OUTLINE_WIDTH 5
-          entries[i].inner_rect.x = OUTLINE_WIDTH;
-          entries[i].inner_rect.y = OUTLINE_WIDTH;
-          entries[i].inner_rect.width = window->rect.width - OUTLINE_WIDTH * 2;
-          entries[i].inner_rect.height = window->rect.height - OUTLINE_WIDTH * 2;
-        }
+      /* Top side */
+      if (!entries[i].hidden &&
+          window->frame && window->frame->bottom_height > 0 &&
+          window->frame->child_y >= window->frame->bottom_height)
+        entries[i].inner_rect.y = window->frame->bottom_height;
+      else
+        entries[i].inner_rect.y = OUTLINE_WIDTH;
+
+      /* Bottom side */
+      if (!entries[i].hidden &&
+          window->frame && window->frame->bottom_height != 0)
+        entries[i].inner_rect.height = r.height
+          - entries[i].inner_rect.y - window->frame->bottom_height;
+      else
+        entries[i].inner_rect.height = r.height
+          - entries[i].inner_rect.y - OUTLINE_WIDTH;
+
+      /* Left side */
+      if (!entries[i].hidden && window->frame && window->frame->child_x != 0)
+        entries[i].inner_rect.x = window->frame->child_x;
+      else
+        entries[i].inner_rect.x = OUTLINE_WIDTH;
+
+      /* Right side */
+      if (!entries[i].hidden &&
+          window->frame && window->frame->right_width != 0)
+        entries[i].inner_rect.width = r.width
+          - entries[i].inner_rect.x - window->frame->right_width;
+      else
+        entries[i].inner_rect.width = r.width
+          - entries[i].inner_rect.x - OUTLINE_WIDTH;
       
       ++i;
       tmp = tmp->next;
@@ -1660,7 +1672,7 @@ meta_screen_update_workspace_layout (MetaScreen *screen)
       meta_XFree (list);
     }
 
-  meta_verbose ("Workspace layout rows = %d cols = %d orientation = %d starting corner = %d\n",
+  meta_verbose ("Workspace layout rows = %d cols = %d orientation = %d starting corner = %u\n",
                 screen->rows_of_workspaces,
                 screen->columns_of_workspaces,
                 screen->vertical_workspaces,
